@@ -7,7 +7,7 @@ import colorsys
 import os
 import copy
 from openpyxl import load_workbook
-from openpyxl.utils import get_column_letter
+from openpyxl.utils import get_column_letter, column_index_from_string
 from openpyxl.styles.colors import RGB, COLOR_INDEX
 from openpyxl.styles.borders import Border
 
@@ -67,6 +67,15 @@ EXPECTED_FAILURES.update({'Extras1.xls': {'Sheet1': {'all': 'image'}}}) # xlrd e
 EXPECTED_FAILURES.update({'Extras2.xls': {'Sheet1': {'all': 'image'}}}) # Real xlsx only shows 1 image, the rest are other things (e.g. text box)
 EXPECTED_FAILURES['timedelta2.mht'] = {'Sheet1': {'all': 'dimension'}}  # Extra row in HTML version
 EXPECTED_FAILURES['Issue4.XLS'] = dict(OUT_ASI_HKG_JPN = dict(all='dimension,width')) # Images at the top get eaten, so the sheet starts at A4, and openpyxl is messing up the column widths
+EXPECTED_FAILURES['Issue5.xls'] = dict(Sheet1 = dict(all='width')) # openpyxl is messing up the column widths
+for ndx in range(column_index_from_string('AK'), column_index_from_string('IV')+1):
+    for r in range(15, 89, 8):
+        EXPECTED_FAILURES['Issue5.xls']['Sheet1'][f'{get_column_letter(ndx)}{r}'] = 'border'
+    EXPECTED_FAILURES['Issue5.xls']['Sheet1'][f'{get_column_letter(ndx)}101'] = 'border'
+    for r in (24, 29, 61, 93, 99):
+        EXPECTED_FAILURES['Issue5.xls']['Sheet1'][f'{get_column_letter(ndx)}{r}'] = 'fill'
+    for r in range(102, 107):
+        EXPECTED_FAILURES['Issue5.xls']['Sheet1'][f'{get_column_letter(ndx)}{r}'] = 'fill'
 
 for col in ('B', 'C', 'N', 'O'):    # Numbers are right-justified in the HTML, but None or 'general' in the xlsx
     for row in range(12, 17):
@@ -100,13 +109,19 @@ EXPECTED_FAILURES['timedelta2.mht']['Sheet1'][f'B7'] = 'number_format'  # Genera
 EXPECTED_FAILURES['timedelta2.mht']['Sheet1'][f'A8'] = 'alignment'  # None vs Right
 EXPECTED_FAILURES['timedelta2.mht']['Sheet1'][f'B8'] = 'value'  # str vs int
 
+EXPECTED_FAILURES['Bad.xls'] = ValueError       # Issue #3
+
 for remote_test in REMOTE_TESTS:
     TESTS.remove(os.path.join(INPUTS, remote_test))
     TESTS.add(f'{URL}/{remote_test}')
 
 color_index = list(COLOR_INDEX)
-color_index[64] = '00000000'      # Index 64: System foreground
-color_index[65] = '00FFFFFF'      # Index 65: System background
+try:
+    color_index[64] = '00000000'      # Index 64: System foreground
+    color_index[65] = '00FFFFFF'      # Index 65: System background
+except Exception:                       # Newer openpyxl deleted these 2 elements
+    color_index.append('00000000')
+    color_index.append('00FFFFFF')
 
 def close_color(rgb1, rgb2):
     """xls colors are mapped using a color palette which doesn't accurately represent the
@@ -126,7 +141,7 @@ def close_color(rgb1, rgb2):
         return True
     yiq1 = colorsys.rgb_to_yiq((rgb1>>24) & 0xff, (rgb1>>16) & 0xff, rgb1 & 0xff)
     yiq2 = colorsys.rgb_to_yiq((rgb2>>24) & 0xff, (rgb2>>16) & 0xff, rgb2 & 0xff)
-    MAX_LUMA_DISTANCE=5
+    MAX_LUMA_DISTANCE=6
     MAX_IN_PHASE_DISTANCE=32
     MAX_QUADRATURE_DISTANCE=55
     if abs(yiq1[0]-yiq2[0]) < MAX_LUMA_DISTANCE and \
@@ -137,6 +152,10 @@ def close_color(rgb1, rgb2):
     return False
 
 def eq(o1, o2):
+    if o1 is None and hasattr(o2, 'style') and o2.style is None:
+        return True     # issue with diagonal: None vs diagonal: Side(style=None)
+    if o2 is None and hasattr(o1, 'style') and o1.style is None:
+        return True     # issue with diagonal: None vs diagonal: Side(style=None)
     if o1 is None or o2 is None:
         return o1 == o2
     if type(o1) is not type(o2):
@@ -153,11 +172,19 @@ def eq(o1, o2):
             return True     # For PatternFill objects, if there is no pattern, ignore both colors
         if o1.patternType == 'solid' and o2.patternType == 'solid':
             ignore_bgColor = True   # If it's a solid fill, we ignore the background color
+        if o1.patternType is None and o2.patternType == 'solid' and o2.fgColor.type == 'rgb' and o2.fgColor.rgb == '00FFFFFF':
+            return True     # White fill === no fill
+        if o2.patternType is None and o1.patternType == 'solid' and o1.fgColor.type == 'rgb' and o1.fgColor.rgb == '00FFFFFF':
+            return True     # White fill === no fill
     if hasattr(o1, 'type') and hasattr(o2, 'type'):
         if o1.type == 'theme' and o2.type == 'rgb' and o1.theme == 1 and o2.rgb == '00000000' and o1.tint == 0.0 and o2.tint == 0.0:
             return True     # Theme color 1 == black
         if o2.type == 'theme' and o1.type == 'rgb' and o2.theme == 1 and o1.rgb == '00000000' and o1.tint == 0.0 and o2.tint == 0.0:
             return True     # Theme color 1 == black
+        if o1.type == 'theme' and o2.type == 'rgb' and o1.theme == 0 and o2.rgb == '00FFFFFF' and o1.tint == 0.0 and o2.tint == 0.0:
+            return True     # Theme color 0 == white
+        if o2.type == 'theme' and o1.type == 'rgb' and o2.theme == 0 and o1.rgb == '00FFFFFF' and o1.tint == 0.0 and o2.tint == 0.0:
+            return True     # Theme color 0 == white
         if o1.type == 'indexed':
             o1 = copy.copy(o1)
             o1.type = 'rgb'
@@ -221,32 +248,32 @@ def or_border(ws1, ws2, row, col):
     ws1_cell = ws1.cell(row, col)
     ws2_cell = ws2.cell(row, col)
     if row != ws1.min_row:
-        if not ws1_cell.border.top.style and ws2_cell.border.top.style and \
-                ws1.cell(row-1, col).border.bottom.style:
+        if (not ws1_cell.border.top or not ws1_cell.border.top.style) and ws2_cell.border.top and ws2_cell.border.top.style and \
+                ws1.cell(row-1, col).border.bottom and ws1.cell(row-1, col).border.bottom.style:
             ws1_cell.border = Border(top=ws1.cell(row-1, col).border.bottom, left=ws1_cell.border.left, right=ws1_cell.border.right, bottom=ws1_cell.border.bottom)
-        elif not ws2_cell.border.top.style and ws1_cell.border.top.style and \
-                ws2.cell(row-1, col).border.bottom.style:
+        elif (not ws2_cell.border.top or not ws2_cell.border.top.style) and ws1_cell.border.top and ws1_cell.border.top.style and \
+                ws2.cell(row-1, col).border.bottom and ws2.cell(row-1, col).border.bottom.style:
             ws2_cell.border = Border(top=ws2.cell(row-1, col).border.bottom, left=ws2_cell.border.left, right=ws2_cell.border.right, bottom=ws2_cell.border.bottom)
     if row != ws1.max_row:
-        if not ws1_cell.border.bottom.style and ws2_cell.border.bottom.style and \
-                ws1.cell(row+1, col).border.top.style:
+        if (not ws1_cell.border.bottom or not ws1_cell.border.bottom.style) and ws2_cell.border.bottom and ws2_cell.border.bottom.style and \
+                ws1.cell(row+1, col).border.top and ws1.cell(row+1, col).border.top.style:
             ws1_cell.border = Border(bottom=ws1.cell(row+1, col).border.top, left=ws1_cell.border.left, right=ws1_cell.border.right, top=ws1_cell.border.top)
-        elif not ws2_cell.border.bottom.style and ws1_cell.border.bottom.style and \
-                ws2.cell(row+1, col).border.top.style:
+        elif (not ws2_cell.border.bottom or not ws2_cell.border.bottom.style) and ws1_cell.border.bottom and ws1_cell.border.bottom.style and \
+                ws2.cell(row+1, col).border.top and ws2.cell(row+1, col).border.top.style:
             ws2_cell.border = Border(bottom=ws2.cell(row+1, col).border.top, left=ws2_cell.border.left, right=ws2_cell.border.right, top=ws2_cell.border.top)
     if col != ws1.min_column:
-        if not ws1_cell.border.left.style and ws2_cell.border.left.style and \
-                ws1.cell(row, col-1).border.right.style:
+        if (not ws1_cell.border.left or not ws1_cell.border.left.style) and ws2_cell.border.left and ws2_cell.border.left.style and \
+                ws1.cell(row, col-1).border.right and ws1.cell(row, col-1).border.right.style:
             ws1_cell.border = Border(left=ws1.cell(row, col-1).border.right, right=ws1_cell.border.right, bottom=ws1_cell.border.bottom, top=ws1_cell.border.top)
-        elif not ws2_cell.border.left.style and ws1_cell.border.left.style and \
-                ws2.cell(row, col-1).border.right.style:
+        elif (not ws2_cell.border.left or not ws2_cell.border.left.style) and ws1_cell.border.left and ws1_cell.border.left.style and \
+                ws2.cell(row, col-1).border.right and ws2.cell(row, col-1).border.right.style:
             ws2_cell.border = Border(left=ws2.cell(row, col-1).border.right, right=ws2_cell.border.right, bottom=ws2_cell.border.bottom, top=ws2_cell.border.top)
     if col != ws1.max_column:
-        if not ws1_cell.border.right.style and ws2_cell.border.right.style and \
-                ws1.cell(row, col+1).border.left.style:
+        if (not ws1_cell.border or not ws1_cell.border.right.style) and ws2_cell.border.right and ws2_cell.border.right.style and \
+                ws1.cell(row, col+1).border.left and ws1.cell(row, col+1).border.left.style:
             ws1_cell.border = Border(right=ws1.cell(row, col+1).border.left, left=ws1_cell.border.left, bottom=ws1_cell.border.bottom, top=ws1_cell.border.top)
-        elif not ws2_cell.border.right.style and ws1_cell.border.right.style and \
-                ws2.cell(row, col+1).border.left.style:
+        elif (not ws2_cell.border.right or not ws2_cell.border.right.style) and ws1_cell.border.right and ws1_cell.border.right.style and \
+                ws2.cell(row, col+1).border.left and ws2.cell(row, col+1).border.left.style:
             ws2_cell.border = Border(right=ws2.cell(row, col+1).border.left, left=ws2_cell.border.left, bottom=ws2_cell.border.bottom, top=ws2_cell.border.top)
 
 best_diff = 0.0
@@ -280,7 +307,14 @@ def test_one_xls(xls):
 
     expected_failures = EXPECTED_FAILURES.get(basename, dict())
 
-    x2x = XLS2XLSX(inp)
+    try:                                    # Issue #3
+        x2x = XLS2XLSX(inp)
+        if expected_failures is ValueError: # Issue #3
+            assert False    # Issue #3: Failed
+    except ValueError as e: # Issue #3
+        if expected_failures is ValueError: # Issue #3
+            return          # Issue #3: Passed
+        raise               # Issue #3: Failed
     x2x.to_xlsx(filename=output)
     wb_exp = load_workbook(filename=expected)
     wb_out = load_workbook(filename=output)
